use Test::More tests => 165 ;

use Stanford::Orange::Util qw/ trim slurp spew tempfile / ;
use Stanford::Certificate::X509 qw/ parse_x509_pem / ;
use Data::Dumper ;

my %cert_field_to_value = () ;

my $pem_root1 = './t/incommon_root.pem' ;
my $pem_root2 = './t/incommon_intermediate.pem' ;
my $pem_root3 = './t/stanford-win-issuer-ca.pem' ;
my $pem_file1 = './t/mdm-test.stanford.edu.pem' ;
my $pem_file2 = './t/opendns.com.pem' ;
my $pem_file3 = './t/verisign_root1.pem' ;
my $pem_file4 = './t/verisign_root3.pem' ;
my $pem_file5 = './t/godaddy_interm.pem' ;
my $pem_file6 = './t/dsa_cert.pem' ;
my $pem_file7 = './t/multi_domain.pem' ;
# my $pem_file8 = './t/verisign_elliptic.pem' ;
my $pem_file9 = './t/self-signed-adamhl.pem' ;
my $pem_file_sha256 = './t/cert_SHA256RSA.pem' ;

my $pem_root_string  ;
my ($sans, %sans) ;
my $key_usage_href ;

# ROOT 1
$pem_root_string = slurp($pem_root1) ;
%cert_field_to_value = parse_x509_pem($pem_root_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq 'C=SE, O=AddTrust AB, OU=AddTrust External TTP Network, CN=AddTrust External CA Root') ;
ok ($cert_field_to_value{'cn'} eq 'AddTrust External CA Root') ;
ok ($cert_field_to_value{'CN'} eq $cert_field_to_value{'cn'}) ;
ok ($cert_field_to_value{'subject_cn'} eq $cert_field_to_value{'cn'}) ;

ok ($cert_field_to_value{'issuer'} eq 'C=SE, O=AddTrust AB, OU=AddTrust External TTP Network, CN=AddTrust External CA Root') ;

ok ($cert_field_to_value{'serial'} eq '1') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq 'ADBD987A34B426F7FAC42654EF03BDE024CB541A') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq 'ADBD987A34B426F7FAC42654EF03BDE024CB541A') ;

ok ($cert_field_to_value{'not_before'} eq '2000-05-30 10:48:38') ;
ok ($cert_field_to_value{'not_after'} eq '2020-05-30 10:48:38') ;
ok ($cert_field_to_value{'CA'} == 1) ;
$key_usage_href = $cert_field_to_value{'key_usage_href'} ;
ok ($key_usage_href->{'keyCertSign'}) ;

ok (!defined($cert_field_to_value{'ocsp_uri'})) ;

ok ($cert_field_to_value{'signature_algorithm'} eq 'sha1WithRSAEncryption');

# ROOT 2
$pem_root_string = slurp($pem_root2) ;
%cert_field_to_value = parse_x509_pem($pem_root_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq 'C=US, O=Internet2, OU=InCommon, CN=InCommon Server CA') ;
ok ($cert_field_to_value{'cn'} eq 'InCommon Server CA') ;
ok ($cert_field_to_value{'CN'} eq $cert_field_to_value{'cn'}) ;
ok ($cert_field_to_value{'subject_cn'} eq $cert_field_to_value{'cn'}) ;


ok ($cert_field_to_value{'serial'} eq '7F71C1D3A226B0D2B113F3E68167643E') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq '484F5AFA2F4A9A5EE050F36B7B55A5DEF5BE345D') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq 'ADBD987A34B426F7FAC42654EF03BDE024CB541A') ;

ok ($cert_field_to_value{'not_before'} eq '2010-12-07 00:00:00') ;
ok ($cert_field_to_value{'not_after'} eq '2020-05-30 10:48:38') ;
ok ($cert_field_to_value{'CA'} == 1) ;
$key_usage_href = $cert_field_to_value{'key_usage_href'} ;
ok ($key_usage_href->{'keyCertSign'}) ;

ok ($cert_field_to_value{'ocsp_uri'} eq 'http://ocsp.usertrust.com') ;

# CERT 1
$pem_root_string = slurp($pem_file1) ;
%cert_field_to_value = parse_x509_pem($pem_root_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq 'C=US, postalCode=94305, ST=CA, L=Stanford, streetAddress=450 Serra Mall, O=Stanford University, OU=IT Services, OU=PlatinumSSL, CN=mdm-test.stanford.edu') ;
ok ($cert_field_to_value{'cn'} eq 'mdm-test.stanford.edu') ;
ok ($cert_field_to_value{'CN'} eq $cert_field_to_value{'cn'}) ;
ok ($cert_field_to_value{'subject_cn'} eq $cert_field_to_value{'cn'}) ;

ok ($cert_field_to_value{'serial'} eq 'C927DF796661923802493DC4DCD3C082') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq '798B26BA217FBE090D5AEE8D023D49D360ECBC41') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq '484F5AFA2F4A9A5EE050F36B7B55A5DEF5BE345D') ;

ok ($cert_field_to_value{'not_before'} eq '2011-05-20 00:00:00') ;
ok ($cert_field_to_value{'not_after'} eq '2014-05-19 23:59:59') ;
ok ($cert_field_to_value{'CA'} == 0) ;

# ROOT 3
$pem_root_string = slurp($pem_root3) ;
%cert_field_to_value = parse_x509_pem($pem_root_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq 'DC=edu, DC=stanford, DC=win, CN=stanford-WIN-Issuer-CA') ;
ok ($cert_field_to_value{'cn'} eq 'stanford-WIN-Issuer-CA') ;
ok ($cert_field_to_value{'CN'} eq $cert_field_to_value{'cn'}) ;
ok ($cert_field_to_value{'subject_cn'} eq $cert_field_to_value{'cn'}) ;

ok ($cert_field_to_value{'serial'} eq '612E3BA9000000000005') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 4096) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq '43F51E2DB82E191B212ABCBC9ED6E6791CD5F581') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq 'B653F00682F1582FA3CBE0590C1D871030B8D838') ;

ok ($cert_field_to_value{'not_before'} eq '2008-03-14 23:20:17') ;
ok ($cert_field_to_value{'not_after'} eq '2013-03-13 23:20:17') ;
ok ($cert_field_to_value{'CA'} == 1) ;
ok ($cert_field_to_value{'ocsp_uri'} eq 'http://itcert.stanford.edu/ocsp') ;

# CERT 2
$pem_file_string = slurp($pem_file2) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq
'serialNumber=u1qcmJQFwrxSq31GeQxJEn9-PfyfU0Fp, C=US, O=*.opendns.com, OU=GT55236522, OU=See www.rapidssl.com/resources/cps (c)10, OU=Domain Control Validated - RapidSSL(R), CN=*.opendns.com') ;
ok ($cert_field_to_value{'cn'} eq '*.opendns.com') ;
ok ($cert_field_to_value{'CN'} eq $cert_field_to_value{'cn'}) ;
ok ($cert_field_to_value{'subject_cn'} eq $cert_field_to_value{'cn'}) ;

ok ($cert_field_to_value{'serial'} eq '142DE9') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 1024) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq '58F368B63721B9CDEA328856378BC3B00D559A5E') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq '48E668F92BD2B295D747D82320104F3398909FD4') ;

ok ($cert_field_to_value{'not_before'} eq '2010-08-04 00:45:38') ;
ok ($cert_field_to_value{'not_after'} eq '2012-09-05 16:34:09') ;
ok ($cert_field_to_value{'CA'} == 0) ;
ok (!defined($cert_field_to_value{'ocsp_uri'})) ;

$sans = $cert_field_to_value{'sans'} ;
%sans = %$sans ;

ok ((0 + (keys %sans)) == 1) ;
ok (exists($sans{'dNSName'})) ;
ok ($sans{'dNSName'}->[0] eq '*.opendns.com') ;
ok ($sans{'dNSName'}->[1] eq 'opendns.com') ;


# CERT 3
$pem_file_string = slurp($pem_file3) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 1) ;
ok ($cert_field_to_value{'subject'} eq
'C=US, O=VeriSign, Inc., OU=Class 3 Public Primary Certification Authority - G2, OU=(c) 1998 VeriSign, Inc. - For authorized use only, OU=VeriSign Trust Network') ;
ok (!defined($cert_field_to_value{'cn'})) ;
ok (!defined($cert_field_to_value{'CN'})) ;
ok (!defined($cert_field_to_value{'subject_cn'})) ;

ok ($cert_field_to_value{'serial'} eq '7DD9FE07CFA81EB7107967FBA78934C6') ;
ok ($cert_field_to_value{'key_size'} eq 1024) ;
ok ($cert_field_to_value{'key_size'} eq $cert_field_to_value{'key_length'}) ;

ok (!defined($cert_field_to_value{'subject_key_identifier'})) ;
ok (!defined($cert_field_to_value{'authority_key_identifier'})) ;

ok ($cert_field_to_value{'not_before'} eq '1998-05-18 00:00:00') ;
ok ($cert_field_to_value{'not_after'} eq '2028-08-01 23:59:59') ;
ok ($cert_field_to_value{'CA'} == 0) ;
ok (!defined($cert_field_to_value{'ocsp_uri'})) ;

ok (!defined($cert_field_to_value{'sans'})) ;



# CERT 4
$pem_file_string = slurp($pem_file4) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;


# CERT 5
$pem_file_string = slurp($pem_file5) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq
'C=US, ST=Arizona, L=Scottsdale, O=GoDaddy.com, Inc., OU=http://certificates.godaddy.com/repository, CN=Go Daddy Secure Certification Authority, serialNumber=07969287') ;

ok ($cert_field_to_value{'cn'} eq 'Go Daddy Secure Certification Authority') ;
ok ($cert_field_to_value{'CN'} eq $cert_field_to_value{'cn'}) ;
ok ($cert_field_to_value{'subject_cn'} eq $cert_field_to_value{'cn'}) ;

ok ($cert_field_to_value{'serial'} eq '301') ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq 'FDAC6132936C45D6E2EE855F9ABAE7769968CCE7') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq 'D2C4B0D291D44C1171B361CB3DA1FEDDA86AD4E3') ;

ok ($cert_field_to_value{'not_before'} eq '2006-11-16 01:54:37') ;
ok ($cert_field_to_value{'not_after'} eq '2026-11-16 01:54:37') ;
ok ($cert_field_to_value{'CA'} == 1) ;
ok ($cert_field_to_value{'ocsp_uri'} eq 'http://ocsp.godaddy.com') ;

ok (!defined($cert_field_to_value{'sans'})) ;


# CERT 6
$pem_file_string = slurp($pem_file6) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq 'C=AU, ST=Some-State, O=Internet Widgits Pty Ltd') ;
ok (!defined($cert_field_to_value{'cn'})) ;

ok ($cert_field_to_value{'serial'} eq 'EB28C269DE7E283F') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'DSA') ;
ok ($cert_field_to_value{'key_size'} eq 1024) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq 'D5A416AD92FC6D4C75A47E20631D322F61876DC1') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq 'D5A416AD92FC6D4C75A47E20631D322F61876DC1') ;

ok ($cert_field_to_value{'not_before'} eq '2011-12-22 16:33:27') ;
ok ($cert_field_to_value{'not_after'} eq '2012-01-21 16:33:27') ;
ok ($cert_field_to_value{'CA'} == 1) ;
ok (!defined($cert_field_to_value{'ocsp_uri'})) ;

ok (!defined($cert_field_to_value{'sans'})) ;

# CERT 7
$pem_file_string = slurp($pem_file7) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 3) ;
ok ($cert_field_to_value{'subject'} eq 'C=US, postalCode=94305, ST=CA, L=Stanford, streetAddress=450 Serra Mall, O=Stanford University, OU=SULAIR, CN=mail.acomp.stanford.edu') ;
ok ($cert_field_to_value{'cn'} eq 'mail.acomp.stanford.edu') ;

ok ($cert_field_to_value{'serial'} eq '48F8C069A20181BA24FB675C60C9781B') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;

ok ($cert_field_to_value{'subject_key_identifier'} eq '9E6C71E9A160DE51CE2BE97BCEF5DEF24E2F9490') ;
ok ($cert_field_to_value{'authority_key_identifier'} eq '484F5AFA2F4A9A5EE050F36B7B55A5DEF5BE345D') ;

ok ($cert_field_to_value{'not_before'} eq '2011-08-24 00:00:00') ;
ok ($cert_field_to_value{'not_after'} eq '2014-08-23 23:59:59') ;
ok ($cert_field_to_value{'CA'} == 0) ;
ok ($cert_field_to_value{'ocsp_uri'} eq 'http://ocsp.incommon.org') ;

$sans = $cert_field_to_value{'sans'} ;
%sans = %$sans ;

# DNS:mail.acomp.stanford.edu
# DNS:mail.rescomp.stanford.edu,
# DNS:mail.stucomp.stanford.edu
# DNS:mx1.acomp.stanford.edu
# DNS:mx2.acomp.stanford.edu
ok (exists($sans{'dNSName'})) ;
ok ($sans{'dNSName'}->[0] eq 'mail.acomp.stanford.edu') ;
ok ($sans{'dNSName'}->[1] eq 'mail.rescomp.stanford.edu') ;
ok ($sans{'dNSName'}->[2] eq 'mail.stucomp.stanford.edu') ;
ok ($sans{'dNSName'}->[3] eq 'mx1.acomp.stanford.edu') ;
ok ($sans{'dNSName'}->[4] eq 'mx2.acomp.stanford.edu') ;


# CERT 9
$pem_file_string = slurp($pem_file9) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'version'} eq 1) ;
ok ($cert_field_to_value{'subject'} eq 'C=US, ST=California, L=Stanford, O=Stanford University, OU=IT Services, CN=self-signed-adamhl.stanford.edu') ;
ok ($cert_field_to_value{'cn'} eq 'self-signed-adamhl.stanford.edu') ;

ok ($cert_field_to_value{'serial'} eq 'FDB68F5361157FAF') ;
ok ($cert_field_to_value{'pubkey_algorithm'} eq 'RSA') ;
ok ($cert_field_to_value{'key_size'} eq 2048) ;

ok (!defined($cert_field_to_value{'subject_key_identifier'})) ;
ok (!defined($cert_field_to_value{'authority_key_identifier'})) ;

ok ($cert_field_to_value{'not_before'} eq '2012-05-09 15:00:44') ;
ok ($cert_field_to_value{'not_after'} eq '2013-05-09 15:00:44') ;

ok (!defined($cert_field_to_value{'sans'})) ;
ok ($cert_field_to_value{'CA'} == 0) ;
ok (!defined($cert_field_to_value{'ocsp_uri'})) ;


# CERT 10
$pem_file_string = slurp($pem_file_sha256) ;
%cert_field_to_value = parse_x509_pem($pem_file_string) ;
ok ($cert_field_to_value{'signature_algorithm'} eq 'sha256WithRSAEncryption');


# Splitting PEM strings.
my @pems ;
my $multi_pem_file0 = './t/empty.pem' ;
my $multi_pem_file1 = './t/one_cert.pem' ;
my $multi_pem_file2 = './t/two_certs.pem' ;
my $multi_pem_file3 = './t/not_a_cert.pem' ;

@pems = Stanford::Certificate::X509::split_pems_string(slurp($multi_pem_file0)) ;
ok (scalar(@pems) == 0) ;

@pems = Stanford::Certificate::X509::split_pems_string(slurp($multi_pem_file1)) ;
ok (scalar(@pems) == 1) ;

@pems = Stanford::Certificate::X509::split_pems_string(slurp($multi_pem_file2)) ;
ok (scalar(@pems) == 2) ;

%cert_field_to_value = parse_x509_pem($pems[0]) ;
ok($cert_field_to_value{'subject'} eq 'C=US, postalCode=94305, ST=CA, L=Stanford, streetAddress=450 Serra Mall, O=Stanford University, OU=IT Services, OU=PlatinumSSL, CN=mdm-test.stanford.edu') ;

%cert_field_to_value = parse_x509_pem($pems[1]) ;
ok($cert_field_to_value{'subject'} eq 'C=SE, O=AddTrust AB, OU=AddTrust External TTP Network, CN=AddTrust External CA Root') ;

eval
{
  @pems = Stanford::Certificate::X509::split_pems_string(slurp($multi_pem_file3)) ;
} ;
ok ($@ =~ m{Found something that is not a certificate}i) ;

# Test the separate_pems function
my $server_cert   = slurp($pem_file1) ;
my $signing_cert1 = slurp($pem_root2) ;
my $signing_cert2 = slurp($pem_root1) ;

my @resulting_pems ;

@pems = ($server_cert, $signing_cert1, $signing_cert2) ;
@resulting_pems = Stanford::Certificate::X509::separate_pems(\@pems) ;
ok (($resulting_pems[0] eq $server_cert) &&
    ($resulting_pems[1] eq $signing_cert1) &&
    ($resulting_pems[2] eq $signing_cert2)
   ) ;

@pems = ($signing_cert1, $server_cert, $signing_cert2) ;
@resulting_pems = Stanford::Certificate::X509::separate_pems(\@pems) ;
ok (($resulting_pems[0] eq $server_cert) &&
    ($resulting_pems[1] eq $signing_cert1) &&
    ($resulting_pems[2] eq $signing_cert2)
   ) ;

@pems = ($signing_cert1, $signing_cert2, $server_cert) ;
@resulting_pems = Stanford::Certificate::X509::separate_pems(\@pems) ;
ok (($resulting_pems[0] eq $server_cert) &&
    ($resulting_pems[1] eq $signing_cert1) &&
    ($resulting_pems[2] eq $signing_cert2)
   ) ;

@pems = ($signing_cert1, $signing_cert2, $signing_cert2) ;
@resulting_pems = Stanford::Certificate::X509::separate_pems(\@pems) ;
ok ((!defined($resulting_pems[0])) &&
    ($resulting_pems[1] eq $signing_cert1) &&
    ($resulting_pems[2] eq $signing_cert2) &&
    ($resulting_pems[2] eq $signing_cert2)
   ) ;

@pems = () ;
@resulting_pems = Stanford::Certificate::X509::separate_pems(\@pems) ;
ok (scalar(@resulting_pems) == 0) ;

@pems = ($signing_cert1, $signing_cert2, $server_cert, $server_cert) ;
eval
{
  @resulting_pems = Stanford::Certificate::X509::separate_pems(\@pems) ;
} ;

ok ($@) ;
