use Test::More tests => 4 ;

use Stanford::Certificate ;
use Stanford::Certificate::Config ;
use Stanford::Certificate::StatusReport ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;
ok ($schema) ;

# Make a fake object
$request = Stanford::Certificate::make_fake_request($schema) ;

# Add a subject
my $cn = 'testingabc.stanford.edu' ;
$request->cn('testingsr7.stanford.edu') ;
$request->make_csr() ;

# Save it.
$request->saveme() ;
$id = $request->id() ;
ok ($id > 0) ;

# Give it an order number
$request->order_number('1234567') ;
$request->update() ;

# Generate a status report

my $status_report = Stanford::Certificate::StatusReport->new() ;
$status_report->schema($schema) ;
$status_report->run_report() ;

my $report = $status_report->report() ;
ok ($report =~ m{Requests\ awaiting\ APPROVAL.*testingsr7\.stanford\.edu}xms) ;

# Approve this order and regenerate
$request->approved(1) ;
$request->update() ;
$status_report->run_report() ;
$report = $status_report->report() ;
ok ($report =~ m{Requests\ awaiting\ COLLECTION.*testingsr7\.stanford\.edu}xms) ;
