use Test::More ;
BEGIN { use_ok('Stanford::Certificate') };

use Stanford::Orange::Util qw/ tempfile / ;
use Stanford::Certificate::X509 ;
use Data::Dumper ;

# Set the DEB_PACKAGE_BUILD environment variable.
require 't/run_non_build_tests.pl' ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

my $request = Stanford::Certificate::make_fake_request($schema) ;

my $pem = tempfile() ;
my $key = tempfile() ;

# Make a random hostname
my @letters=(0..9);
my $total = @letters ;
my $random_suffix = q{} ;
for (my $i=1; $i <= 5; ++$i)
{
  $random_suffix .= $letters[rand $total] ;
}
my $hostname = "ahltest" . $random_suffix ;
my $cn = "${hostname}.stanford.edu" ;
my $csr_string = Stanford::Certificate::X509::make_csr($cn, $pem, $key) ;

unlink($pem) ;
unlink($key) ;
ok (length($csr_string) > 100) ;

my $sunetid       = 'adamhl' ;
my $fullname      = 'Adam Henry Lewenberg' ;
my $email         = 'adamhl@stanford.edu' ;
my $affiliation   = 'Staff' ;
my $department    = 'IT Services' ;
my $server_name   = $hostname ;
my $server_type   = 'Apache' ;
my $contact_email = 'adamhl@stanford.edu' ;
my $duration      = 3 ;

my $tools_request = $request->create_from_web_page(
  'requester_sunetid'     => 'adamhl',
  'requester_fullname'    => 'Adam Henry Lewenberg',
  'requester_email'       => 'adamhl@stanford.edu',
  'requester_affiliation' => 'Staff',
  'requester_department'  => 'IT Services',
  'cn'                    => $cn,
  'server_type_name'      => 'Apache/ModSSL',
  'contact_email'         => 'adamhl@stanford.edu',
  'duration'              => 3,
  'csr'                   => $csr_string,
  'reissue_boolean'       => 0,
  'approval_contacts'     => 'adamhl@stanford.edu',
) ;

$tools_request->test(1) ;

ok (length($tools_request->csr()) > 100) ;
$tools_request->insert() ;
ok ($tools_request->id() =~ m{^\d+$}) ;

my $number_of_tests_run ;

# Finally, let's actually _send_ the request (but not when building the
# package)
SKIP:
{
  if (defined($ENV{'DEB_PACKAGE_BUILD'}) && (defined($ENV{'DEB_PACKAGE_BUILD'})eq '1'))
  {
    skip "skipping request send and approver notify", 2 ;
  }

  my $cm_credentials_yml_file = $CONFIG->incommon_cm_credentials_yaml_file() ;
  skip "skipping request send", 2 if (!(-e $cm_credentials_yml_file)) ;

  ok ($tools_request->send($cm_credentials_yml_file)) ;

  # Send the notice to the approvers.
  $tools_request->notify_approvers() ;

  # Notify the interested parties that the
  # request has been sent to InCommon and that we
  # have an order number.
  ok ($tools_request->notify_sent()) ;
}

done_testing( $number_of_tests_run ) ;