use Test::More tests => 8 ;

use Stanford::Certificate ;

use Date::Calc::Object ;
use Stanford::Certificate::Notification ;
use Data::Dumper ;

# Set the DEB_PACKAGE_BUILD environment variable.
require 't/run_non_build_tests.pl' ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

# Because the time zone inside git-buildpackage is UTC (i.e., GMT), set
# it explicitly here.
$ENV{'TZ'} ='America/Los_Angeles' ;

my $legacy_expiration = Stanford::Certificate::Result::LegacyExpiration::make_fake($schema) ;
ok ($legacy_expiration) ;

# Make two legacy expirations, one expiring tomorrow, one expiring in 7
# days, one hour, and one expiring in 7 days 2 hours, and one expiring in
# 8 days 1 hour.

sub pad
{
  my ($n) = @_;
  if ($n + 0 < 10) { return '0' . $n ; }
  else             { return q{} . $n ; }
}

my $date_formatter_fref = sub
{
  my($self,$code,$lang) = @_;

  my $year  =     $self->year() ;
  my $month = pad($self->month()) ;
  my $day   = pad($self->day()) ;
  return qq{$year-$month-$day} ;
} ;

# Remember that LegacyExpiration objects are dates only (no times).
sub make_fake_leg_exp
{
  my ($dc_fixed, $offset_array_ref) = @_ ;
  my $dc = $dc_fixed->clone() ;
  $dc += $offset_array_ref ;

  my $le = Stanford::Certificate::Result::LegacyExpiration::make_fake($schema) ;

  my $date = $dc->string($date_formatter_fref) ;
  $le->expires_at($date) ;

  $le->insert() ;
  return $le ;
}


my %legacy_expirations = () ;

# Pretend that today is 30-June-2012 13:05:07
my $dc_fixed = Date::Calc->new([2012, 6, 30, 13, 05, 07]) ;
my $date_string_fixed = $dc_fixed->string($date_formatter_fref) ;

# Only the two ones marked below expire in [7, 8) days from $dc_fixed
# (remember that legacy expirations are strict at the upper limit).
$legacy_expirations{'tomorrow'}    = make_fake_leg_exp($dc_fixed, [0,0,1,1,0,0]) ;
$legacy_expirations{'7days1hour'}  = make_fake_leg_exp($dc_fixed, [0,0,7,1,0,0]) ; # This one
$legacy_expirations{'7days2hours'} = make_fake_leg_exp($dc_fixed, [0,0,7,2,0,0]) ; # and this one
$legacy_expirations{'8days1hour'}  = make_fake_leg_exp($dc_fixed, [0,0,8,1,0,0]) ;

my @legacy_expirations ;
@legacy_expirations =
  Stanford::Certificate::Notification::get_expiring_legacy_expirations(
  $schema, $dc_fixed, 7, 8
) ;

ok (scalar(@legacy_expirations) == 2) ;

@legacy_expirations =
  Stanford::Certificate::Notification::get_expiring_legacy_expirations(
  $schema, $dc_fixed, 8, 9
) ;
ok (scalar(@legacy_expirations) == 1) ;

# Suppress notification on one:
$legacy_expirations{'7days1hour'}->suppress_notifications(1) ;
$legacy_expirations{'7days1hour'}->update() ;
@legacy_expirations =
  Stanford::Certificate::Notification::get_expiring_legacy_expirations(
  $schema, $dc_fixed, 7, 8
) ;

ok (scalar(@legacy_expirations) == 1) ;

# And now the other one:
$legacy_expirations{'7days2hours'}->suppress_notifications(1) ;
$legacy_expirations{'7days2hours'}->update() ;
@legacy_expirations =
  Stanford::Certificate::Notification::get_expiring_legacy_expirations(
  $schema, $dc_fixed, 7, 8
) ;

ok (scalar(@legacy_expirations) == 0) ;


# Make one expire

SKIP:
{
  if (defined($ENV{'DEB_PACKAGE_BUILD'}) && (defined($ENV{'DEB_PACKAGE_BUILD'}) eq '1'))
  {
    skip "skipping e-mail send", 0 ;
  }

  Stanford::Certificate::Notification::email_expiring_legacy_expirations(
    \@legacy_expirations) ;
}

# Make a legacy expiration, then make a new certificate with the same
# common name, then run
# suppress_superseded_legacy_expirations and ensure that the legacy
# expiration was marked to suppress notification.
my $cn = 'certreq-sbx.stanford.edu' ;
my $legexp = $schema->resultset('LegacyExpiration')->new({});

$legexp->contact_email('leland.test@mailtonews.stanford.edu') ;
$legexp->expires_at('2012-12-25') ;
$legexp->cn($cn) ;
$legexp->insert() ;

my $legexp_id = $legexp->id() ;
ok ($legexp_id =~ m{^\d+$}) ;
ok ($legexp->suppress_notifications() == 0) ;

my $cert = $schema->resultset('Certificate')->new({});
my $new_cert = $cert->make_cert_from_pem_file('./t/certreq-sbx.stanford.edu.pem') ;

Stanford::Certificate::Notification::suppress_superseded_legacy_expirations($schema, $cn) ;
my $legexp1 = $legexp->findById($legexp_id) ;
ok ($legexp1->suppress_notifications() == 1) ;
