use strict ;
use warnings ;

# Create the base schema file. Should be run on package installation.

# Use:
#  ./stanford_cert_make_schema.pl <environment>
#  ./stanford_cert_make_schema.pl sbx   (example)

## ##########################################################

## Create a DBIx::Class-compatible schema from a live database.
use DBIx::Class::Schema::Loader qw/ make_schema_at /;

use lib qw( ../lib lib ) ;
use Stanford::Certificate::Misc ;

my $connect_yml_file = Stanford::Certificate::Misc::get_connect_yml_file() ;
print "connect_yml_file is $connect_yml_file\n" ;

# Create a YAML object from file
use YAML::Tiny;
my $yaml = YAML::Tiny->read($connect_yml_file);

# Read
my $dsn      = $yaml->[0]->{'perl_dsn'};
my $username = $yaml->[0]->{'username'};
my $password = $yaml->[0]->{'password'};

if (!$dsn)
{
  die "no DSN found!\n" ;
}

## STEP 2: Make the schema and put it in ../lib
#  skip_load_external is important!
make_schema_at(
    'Stanford::Certificate',
    { debug => 1,
      dump_directory     => '../lib',
      skip_load_external => 1,
      #        dump_directory => '/tmp',
      filter_generated_code => sub {
          return "#<<<\n## no critic\n$_[2]\n## use critic\n#>>>";
      },
    },
    [ $dsn, $username, $password,
      # { loader_class => 'MyLoader' } # optionally
    ],
    # Put perltidy markers so perltidy will ignore the autogenerated
    # part of the class.
    );

